# hyperdex Optimization

This document explains the optimization applied to the repository fetching process in the hyperdex scripts.

## Terminal Testing First: A Critical Practice

**All optimizations in this project began with direct terminal testing before implementation.**

This crucial practice cannot be overstated: **ALWAYS test your approach directly in the terminal before scripting.**

### Testing Endpoint Selection for Private Repositories

Before modifying the script to include private repositories, we tested different approaches in the terminal:

```bash
echo "CURRENT APPROACH (users endpoint - only public repos):" && \
gh api "users/shaneholloman/repos?per_page=5&type=owner" | \
jq '[.[] | {name: .name, private: .private}]' && \
echo -e "\nPROPOSED SOLUTION (user repos with affiliation=owner - includes private repos):" && \
gh api "user/repos?per_page=5&affiliation=owner" | \
jq '[.[] | {name: .name, private: .private}]'
```

Then we specifically tested for private repositories to confirm our approach:

```bash
echo "TEST FOR PRIVATE REPOS:" && \
gh api "user/repos?affiliation=owner&visibility=private&per_page=5" | \
jq '[.[] | {name: .name, private: .private}]'
```

This testing confirmed that:

1. The `users/{username}/repos` endpoint returns only public repositories
2. The `user/repos` endpoint with `affiliation=owner` returns both public and private repositories
3. The `visibility=private` parameter can be used to specifically filter for private repositories

Only after confirming our approach in the terminal did we proceed to implement it in the script.

The GraphQL optimization described below was discovered and refined through this process:

1. First, testing basic GitHub API calls to understand the original approach:

    ```bash
    gh api users/shaneholloman/repos
    gh api repos/CelloCommunications/repo-name/commits
    ```

2. Then, exploring GraphQL alternatives directly in the terminal:

    ```bash
    gh api graphql -f query='query { organization(login: "CelloCommunications") { ... } }'
    ```

3. Refining and testing the queries interactively:

    ```bash
    gh api graphql -f query='...' | jq '.data.organization.repositories.nodes[]'
    ```

Only after we confirmed that our approach worked in the terminal and understood the response structure did we proceed to implement it in Python code.

**Benefits of terminal testing first:**

- Immediate feedback on API responses
- Faster iteration on approach
- Clear understanding of data structures
- Identification of issues before coding
- Confidence in the implementation approach

This approach saved significant development time and led directly to the optimizations described below.

## Original Approach vs. Optimized Approach

### Original Approach in `github_readme_fetcher_cello.py`

The original implementation used a two-step process:

1. Fetch all repositories from the organization (1 API call)
2. For each repository, make a separate API call to check authorship (N API calls)

This resulted in a total of **1 + N** API calls, where N is the number of repositories in the organization.

```sh
Repositories → API Call 1
├── Repo 1 → API Call for authorship check
├── Repo 2 → API Call for authorship check
├── Repo 3 → API Call for authorship check
└── ... (and so on for each repository)
```

For large organizations with hundreds of repositories, this could easily approach GitHub's API rate limits.

### Optimized Approach (GraphQL Implementation)

The optimized version uses GitHub's GraphQL API to combine repository listing and authorship checking into a single query:

1. Fetch repositories with authorship information in a single paginated query

This results in as few as **1 API call** for the entire operation (or a few calls if pagination is necessary).

```sh
GraphQL Query → Single API Call
├── Returns repositories AND authorship data together
└── No additional API calls needed per repository
```

## Technical Implementation

The key to the optimization is the GraphQL query that fetches both repository metadata and the first commit author in a single request:

```graphql
query {
  organization(login: "OrganizationName") {
    repositories(first: 100) {
      pageInfo {
        hasNextPage
        endCursor
      }
      nodes {
        name
        isFork
        description
        createdAt
        updatedAt
        defaultBranchRef {
          target {
            ... on Commit {
              history(first: 1) {
                nodes {
                  author {
                    user {
                      login
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
```

This query:

- Fetches basic repository information (name, description, dates)
- Simultaneously retrieves the first commit author
- Supports pagination for handling large organizations
- Returns all needed data in a structured format

## Performance Benefits

Testing with the CelloCommunications organization demonstrated significant improvements:

| Metric | Original Approach | GraphQL Approach |
|--------|------------------|-----------------|
| API Calls | 100+ (with ~100 repos) | 1 |
| Rate Limit Usage | ~2% of hourly limit | ~0.02% of hourly limit |
| Script Runtime | Proportional to repo count | Nearly constant |

For the specific case tested:

- Found 20 repositories authored by shaneholloman
- Required only 1 API call (instead of 100+)
- Maintained identical output format and functionality

## Implementation Considerations

When implementing GraphQL for GitHub API interactions:

1. Pagination handling is essential for large organizations
2. Error handling is important for repositories with missing history data
3. The GraphQL query can be customized to fetch additional metadata as needed
4. GitHub's rate limits are much less likely to be a concern with this approach
