# README Management Process

This document outlines the process for managing README files across multiple repositories, including fixing markdown linting issues and pushing updates to GitHub repositories.

## Overview

The hyperdex tool helps manage README files for multiple repositories. Sometimes these README files need to be updated to fix formatting issues or to add new content. This document describes a streamlined process for:

1. Identifying markdown linting issues
2. Fixing those issues locally
3. Pushing the updated README files to their respective repositories

## Markdown Linting Standards

We follow these markdown linting standards for all README files:

- **MD031**: Fenced code blocks should be surrounded by blank lines
- **MD040**: Fenced code blocks should have a language specified
- **MD047**: Files should end with a single newline character
- Use `sh` instead of `bash` for shell code blocks for consistency

## Process for Updating README Files

### 1. Identify Repositories Needing Updates

First, identify which repositories need README updates. This can be done by:

- Running markdown linting tools against the README files
- Visual inspection for formatting issues
- Checking for outdated content

### 2. Fix Markdown Linting Issues Locally

For each README file that needs updating:

1. Add blank lines around code blocks (MD031)
2. Add language specification to code blocks that don't have one (MD040)
3. Change `bash` to `sh` in code fence specifications for consistency
4. Ensure the file ends with a single newline (MD047)

Example of fixing a code block:

```diff
- 1. Clone the repository:
-    ```bash
-    git clone https://github.com/username/repo.git
-    cd repo
-    ```
+ 1. Clone the repository:
+
+    ```sh
+    git clone https://github.com/username/repo.git
+    cd repo
+    ```
```

### 3. Push Updates to GitHub Repositories

To push the updated README files to their respective repositories, use the GitHub CLI with the following command pattern:

```sh
# First, get the SHA of the current README file
sha=$(gh api repos/username/repo/contents/README.md | jq -r '.sha')

# Then, update the README file with the new content
gh api -X PUT /repos/username/repo/contents/README.md \
    -f message="Fix markdown formatting" \
    -f content="$(base64 < path/to/local/readme.md)" \
    -f sha="$sha"
```

This approach:

- Gets the SHA of the current file (required for updating existing files)
- Uses the GitHub API to update the file
- Base64 encodes the file content
- Provides a commit message

## Real-World Example

We recently updated six README files using this process:

1. **ai-experiments**: <https://github.com/shaneholloman/ai-experiments>
2. **aria**: <https://github.com/shaneholloman/aria>
3. **echo**: <https://github.com/shaneholloman/echo>
4. **iris**: <https://github.com/shaneholloman/iris>
5. **oracle**: <https://github.com/shaneholloman/oracle>
6. **reactflow-test**: <https://github.com/shaneholloman/reactflow-test>

### Step-by-Step Example

For the `ai-experiments` repository:

1. First, we identified markdown linting issues:
    - Missing blank lines around code blocks
    - Some code blocks without language specification
    - Missing trailing newline

2. We fixed these issues in the local copy of the README file.

3. We pushed the updated README to GitHub:

    ```sh
    # Get the SHA of the current README file
    sha=$(gh api repos/shaneholloman/ai-experiments/contents/README.md | jq -r '.sha')

    # Update the README file with the new content
    gh api -X PUT /repos/shaneholloman/ai-experiments/contents/README.md \
        -f message="Fix markdown formatting" \
        -f content="$(base64 < github-project-readmes-shane/ai-experiments-readme.md)" \
        -f sha="$sha"
    ```

4. We repeated this process for each repository.

## Automation Opportunities

This process could be further automated with a script that:

1. Loops through a list of repositories
2. Gets the SHA of each README file
3. Updates each README file with the fixed content
4. Provides a summary of the updates

This process has been automated with the `tools/gh_repo_update_readmes.py` Python tool. This tool reads repository names from a specified CSV file (like the one generated by `tools/gh_repo_fetch_index_shane.py`) and updates the corresponding remote READMEs from local files.

Example usage:

```sh
# Default usage (reads from github-project-readmes-shane/repositories.csv)
python3 tools/gh_repo_update_readmes.py

# Specify CSV file and local README directory
python3 tools/gh_repo_update_readmes.py \
    -c path/to/your/repositories.csv \
    -d path/to/your/local/readmes/ \
    -u your_github_username
```

This Python tool provides more robust error handling and flexibility compared to the previous shell script.

## Best Practices

1. **Always get the current SHA**: The GitHub API requires the SHA of the current file when updating existing files.
2. **Use meaningful commit messages**: Clearly describe what changes were made.
3. **Test locally first**: Verify that your markdown fixes look correct before pushing.
4. **Batch similar changes**: Group similar types of updates together for efficiency.
5. **Document the process**: Keep track of what repositories were updated and what changes were made.
